@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000, 2001, 2002 Marco Pesenti Gritti
+ *  Copyright © 2000, 2001, 2002 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,9 +13,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-embed-dialog.h,v 1.4 2004/01/01 22:54:10 chpe Exp $
+ *  $Id$
  */
 
 #ifndef EPHY_EMBED_DIALOG_H
@@ -37,11 +37,11 @@ G_BEGIN_DECLS
 #define EPHY_IS_EMBED_DIALOG_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), EPHY_TYPE_EMBED_DIALOG))
 #define EPHY_EMBED_DIALOG_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), EPHY_TYPE_EMBED_DIALOG, EphyEmbedDialogClass))
 
-typedef struct EphyEmbedDialogClass	EphyEmbedDialogClass;
-typedef struct EphyEmbedDialog		EphyEmbedDialog;
-typedef struct EphyEmbedDialogPrivate	EphyEmbedDialogPrivate;
+typedef struct _EphyEmbedDialogClass	EphyEmbedDialogClass;
+typedef struct _EphyEmbedDialog		EphyEmbedDialog;
+typedef struct _EphyEmbedDialogPrivate	EphyEmbedDialogPrivate;
 
-struct EphyEmbedDialog
+struct _EphyEmbedDialog
 {
         EphyDialog parent;
 
@@ -49,7 +49,7 @@ struct EphyEmbedDialog
         EphyEmbedDialogPrivate *priv;
 };
 
-struct EphyEmbedDialogClass
+struct _EphyEmbedDialogClass
 {
         EphyDialogClass parent_class;
 };
