@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ContentHandler.cpp 7967 2008-02-18 13:40:59Z chpe $
+ *  $Id$
  */
 
 #include "mozilla-config.h"
@@ -35,6 +35,7 @@
 
 #include <nsStringAPI.h>
 
+#include <nsAutoPtr.h>
 #include <nsCExternalHandlerService.h>
 #include <nsComponentManagerUtils.h>
 #include <nsIDOMWindow.h>
@@ -66,7 +67,10 @@
  * content handler.
  */
 GContentHandler::GContentHandler()
-: mUserTime(0)
+: mHelperApp(NULL),
+  mPermission(EPHY_MIME_PERMISSION_UNKNOWN),
+  mUserTime(0),
+  mAction(CONTENT_ACTION_NONE)
 {
 	LOG ("GContentHandler ctor (%p)", this);
 }
@@ -84,6 +88,8 @@ GContentHandler::Show (nsIHelperAppLauncher *aLauncher,
 		       nsISupports *aContext,
 		       PRUint32 aReason)
 {
+        nsRefPtr<GContentHandler> kungFuDeathGrip(this);
+
 	nsresult rv;
 	EphyEmbedSingle *single;
 	gboolean handled = FALSE;
@@ -129,6 +135,9 @@ NS_IMETHODIMP GContentHandler::PromptForSaveToFile(
 				    nsISupports *aWindowContext,
 				    const PRUnichar *aDefaultFile,
 				    const PRUnichar *aSuggestedFileExtension,
+#ifdef HAVE_GECKO_1_9
+				    PRBool aForcePrompt,
+#endif
 				    nsILocalFile **_retval)
 {
 	EphyFileChooser *dialog;
@@ -220,10 +229,10 @@ NS_METHOD GContentHandler::Init ()
 	return NS_OK;
 }
 
-static void
-response_cb (GtkWidget *dialog,
-	     int response,
-	     GContentHandler *self)
+/* static */ void
+GContentHandler::ResponseCallback (GtkWidget *dialog,
+                                   int response,
+                                   GContentHandler *self)
 {
 	gtk_widget_destroy (dialog);
 
@@ -345,7 +354,7 @@ NS_METHOD GContentHandler::MIMEConfirmAction ()
 
 	NS_ADDREF_THIS();
 	g_signal_connect_data (dialog, "response",
-			       G_CALLBACK (response_cb), this,
+			       G_CALLBACK (ResponseCallback), this,
 			       (GClosureNotify) release_cb, (GConnectFlags) 0);
 
 	/* FIXME: should find a way to get the user time of the user action which
@@ -364,7 +373,7 @@ NS_METHOD GContentHandler::MIMEInitiateAction (void)
 
 	auto_downloads = eel_gconf_get_boolean (CONF_AUTO_DOWNLOADS);
 
-	mHelperApp = g_app_info_get_default_for_type (mMimeType.get(), TRUE);
+	mHelperApp = g_app_info_get_default_for_type (mMimeType.get(), FALSE);
 	mPermission = ephy_file_check_mime (mMimeType.get());
 
 	/* HACK! Check that this 'helper application' isn't Epiphany itself,
@@ -381,11 +390,13 @@ NS_METHOD GContentHandler::MIMEInitiateAction (void)
 		}
 	}
 
+#ifndef HAVE_GECKO_1_9
 	if (auto_downloads)
 	{
 		mAction = CONTENT_ACTION_OPEN;
 	}
 	else
+#endif /* !HAVE_GECKO_1_9 */
 	{
 		mAction = CONTENT_ACTION_OPEN_TMP;
 	}
@@ -410,12 +421,13 @@ NS_METHOD GContentHandler::MIMEInitiateAction (void)
 NS_METHOD GContentHandler::MIMEDoAction (void)
 {
 	/* This is okay, since we either clicked on a button, or we get 0 */
-	mUserTime = gtk_get_current_event_time ();
+	mUserTime = (PRUint32) gtk_get_current_event_time ();
 
 	nsCOMPtr<nsIMIMEInfo> mimeInfo;
 	mLauncher->GetMIMEInfo(getter_AddRefs(mimeInfo));
 	NS_ENSURE_TRUE (mimeInfo, NS_ERROR_FAILURE);
 
+#ifndef HAVE_GECKO_1_9
 	char *info = NULL;
 
 	if (mAction == CONTENT_ACTION_OPEN)
@@ -435,8 +447,6 @@ NS_METHOD GContentHandler::MIMEDoAction (void)
 		info = g_strdup_printf ("gnome-browse-to-file:%d", gtk_get_current_event_time());
 	}
 
-	/* See http://bugzilla.gnome.org/show_bug.cgi?id=456945 */
-#ifndef HAVE_GECKO_1_9
 	if (info != NULL)
 	{
 		nsString desc;
@@ -455,6 +465,8 @@ NS_METHOD GContentHandler::MIMEDoAction (void)
 	}
 #endif /* HAVE_GECKO_1_9 */
 
+        nsRefPtr<GContentHandler> kungFuDeathGrip(this);
+
 	if (mAction == CONTENT_ACTION_OPEN)
 	{
 		mLauncher->SaveToDisk (nsnull, PR_FALSE);
@@ -472,5 +484,20 @@ NS_METHOD GContentHandler::MIMEDoAction (void)
 		mLauncher->SaveToDisk (nsnull, PR_FALSE);
 	}
 
+#ifdef HAVE_GECKO_1_9
+        /* We have to do this work down here because the external helper app modifies the
+         * value after calling SaveToDisk.
+         */
+	nsHandlerInfoAction action;
+	if (mAction == CONTENT_ACTION_DOWNLOAD) {
+		action = EPHY_ACTION_BROWSE_TO_FILE;
+
+                /* This won't be able to transport the activation time so we cannot
+                * do startup notification, but it's the best that was available
+                */
+                mimeInfo->SetPreferredAction (action);
+        }
+#endif
+
 	return NS_OK;
 }
