@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: EphyBrowser.h 7896 2008-01-28 20:37:51Z cosimoc $
+ *  $Id$
  */
 
 #ifndef EPHY_BROWSER_H
@@ -37,6 +37,7 @@
 #include <nsIWebBrowser.h>
 #include <nsIWebNavigation.h>
 #include <nsPIDOMWindow.h>
+#include <nsIURIFixup.h>
 
 #ifdef ALLOW_PRIVATE_API
 #include <nsIContentViewer.h>
@@ -107,6 +108,18 @@ public:
 	EphyDOMScrollEventListener(EphyBrowser *aOwner) : EphyEventListener(aOwner) { };
 };
 
+#ifdef HAVE_GECKO_1_9
+class EphyCommandEventListener : public EphyEventListener
+{
+public:
+	NS_IMETHOD HandleEvent(nsIDOMEvent* aEvent);
+
+	EphyCommandEventListener(EphyBrowser *aOwner) : EphyEventListener(aOwner) { };
+};
+#else
+class EphyCommandEventListener;
+#endif /* HAVE_GECKO_1_9 */
+
 class EphyContextMenuListener : public nsIDOMContextMenuListener
 {
 public:
@@ -131,6 +144,7 @@ friend class EphyMiscDOMEventsListener;
 friend class EphyDOMScrollEventListener;
 friend class EphyPopupBlockEventListener;
 friend class EphyModalAlertEventListener;
+friend class EphyCommandEventListener;
 friend class EphyContextMenuListener;
 public:
 	EphyBrowser();
@@ -204,11 +218,15 @@ private:
 	EphyDOMScrollEventListener *mDOMScrollEventListener;
 	EphyPopupBlockEventListener *mPopupBlockEventListener;
 	EphyModalAlertEventListener *mModalAlertListener;
+#ifdef HAVE_GECKO_1_9
+	EphyCommandEventListener *mCommandEventListener;
+#endif
 	EphyContextMenuListener *mContextMenuListener;
 	PRBool mInitialized;
 #ifdef HAVE_MOZILLA_PSM
 	nsCOMPtr<nsISecureBrowserUI> mSecurityInfo;
 #endif
+        nsCOMPtr<nsIURIFixup> mFixup;
 
 	nsresult GetListener (void);
 	nsresult AttachListeners (void);
