@@ -19,13 +19,13 @@
  *
  *  You should have received a copy of the GNU Library General Public
  *  License along with this library; if not, write to the
- *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- *  Boston, MA 02111-1307, USA.
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *  Boston, MA 02110-1301, USA.
  *
- *  $Id: plugin.cpp,v 1.6 2006/09/13 11:34:21 chpe Exp $
+ *  $Id$
  */
 
-#include "mozilla-config.h"
+#include <xpcom-config.h>
 #include "config.h"
 
 #include <string.h>
@@ -34,6 +34,8 @@
 #include <gtk/gtkwindow.h>
 #include <gtk/gtkmessagedialog.h>
 
+#include "ephy-stock-icons.h"
+
 #include <npupp.h>
 #include <nsCOMPtr.h>
 #include <nsIDOMWindow.h>
@@ -196,7 +198,7 @@ show_error_dialog (NPP instance,
 
 	nsCOMPtr<nsIDOMWindow> domWin;
 	mozilla_functions.getvalue (instance, NPNVDOMWindow,
-				    NS_STATIC_CAST (nsIDOMWindow **, getter_AddRefs (domWin)));
+				    static_cast<nsIDOMWindow **>(getter_AddRefs (domWin)));
 	parent = EphyUtils::FindGtkParent (domWin);
 
 	dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
@@ -206,7 +208,7 @@ show_error_dialog (NPP instance,
 
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 						  secondary_text);
-	gtk_window_set_icon_name (GTK_WINDOW (dialog), "web-browser");
+	gtk_window_set_icon_name (GTK_WINDOW (dialog), EPHY_STOCK_EPHY);
 
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
