@@ -15,9 +15,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-bookmark-properties.c 6830 2007-01-18 12:36:08Z lferrett $
+ *  $Id$
  */
 
 #include "config.h"
@@ -40,6 +40,7 @@
 
 
 #include <glib/gi18n.h>
+#include <gtk/gtk.h>
 #include <gtk/gtkcheckbutton.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkstock.h>
@@ -79,7 +80,7 @@ enum
 	PROP_CREATING
 };
 
-static GObjectClass *parent_class;
+G_DEFINE_TYPE (EphyBookmarkProperties, ephy_bookmark_properties, GTK_TYPE_DIALOG)
 
 static gboolean
 update_warning (EphyBookmarkProperties *properties)
@@ -505,8 +506,9 @@ ephy_bookmark_properties_constructor (GType type,
 	const char *tmp;
 	char *text;
 
-	object = parent_class->constructor (type, n_construct_properties,
-					    construct_params);
+	object = G_OBJECT_CLASS (ephy_bookmark_properties_parent_class)->constructor (type,
+                                                                                      n_construct_properties,
+                                                                                      construct_params);
 
 	widget = GTK_WIDGET (object);
 	window = GTK_WINDOW (object);
@@ -515,7 +517,7 @@ ephy_bookmark_properties_constructor (GType type,
 	priv = properties->priv;
 
 	gtk_window_set_icon_name (window, EPHY_STOCK_BOOKMARK);
-	gtk_window_set_type_hint (window, GDK_WINDOW_TYPE_HINT_NORMAL);
+	gtk_window_set_type_hint (window, GDK_WINDOW_TYPE_HINT_DIALOG);
 
 	g_signal_connect (properties, "response",
 			  G_CALLBACK (bookmark_properties_response_cb), properties);
@@ -548,7 +550,7 @@ ephy_bookmark_properties_constructor (GType type,
 	gtk_widget_show (table);
 
 	entry = gtk_entry_new ();
-	gtk_entry_set_editable (GTK_ENTRY (entry), !lockdown);
+	gtk_editable_set_editable (GTK_EDITABLE (entry), !lockdown);
 	tmp = ephy_node_get_property_string (properties->priv->bookmark,
 					     EPHY_NODE_BMK_PROP_TITLE);
 	gtk_entry_set_text (GTK_ENTRY (entry), tmp);
@@ -565,7 +567,7 @@ ephy_bookmark_properties_constructor (GType type,
 	gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 0, 1, GTK_FILL | GTK_EXPAND, 0, 0, 0);
 
 	entry = gtk_entry_new ();
-	gtk_entry_set_editable (GTK_ENTRY (entry), !lockdown);
+	gtk_editable_set_editable (GTK_EDITABLE (entry), !lockdown);
 	tmp = ephy_node_get_property_string (properties->priv->bookmark,
 					     EPHY_NODE_BMK_PROP_LOCATION);
 	gtk_entry_set_text (GTK_ENTRY (entry), tmp);
@@ -581,7 +583,7 @@ ephy_bookmark_properties_constructor (GType type,
 	gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 1, 2, GTK_FILL | GTK_EXPAND, 0, 0, 0);
 
 	entry = ephy_topics_entry_new (priv->bookmarks, priv->bookmark);
-	gtk_entry_set_editable (GTK_ENTRY (entry), !lockdown);
+	gtk_editable_set_editable (GTK_EDITABLE (entry), !lockdown);
 	priv->entry = entry;
 	gtk_widget_show (entry);
 	label = gtk_label_new_with_mnemonic(_("T_opics:"));
@@ -730,8 +732,6 @@ ephy_bookmark_properties_class_init (EphyBookmarkPropertiesClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->constructor = ephy_bookmark_properties_constructor;
 	object_class->set_property = ephy_bookmark_properties_set_property;
 	object_class->get_property = ephy_bookmark_properties_get_property;
@@ -764,34 +764,6 @@ ephy_bookmark_properties_class_init (EphyBookmarkPropertiesClass *klass)
 
 /* public API */
 
-GType
-ephy_bookmark_properties_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (EphyBookmarkPropertiesClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) ephy_bookmark_properties_class_init,
-			NULL,
-			NULL,
-			sizeof (EphyBookmarkProperties),
-			0,
-			(GInstanceInitFunc) ephy_bookmark_properties_init
-		};
-
-		type = g_type_register_static (GTK_TYPE_DIALOG,
-					       "EphyBookmarkProperties",
-					       &our_info, 0);
-	}
-
-	return type;
-}
-
 GtkWidget *
 ephy_bookmark_properties_new (EphyBookmarks *bookmarks,
 			      EphyNode *bookmark,
