@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-bookmarks.c 7687 2007-11-13 23:15:57Z xan $
+ *  $Id$
  */
 
 #include "config.h"
@@ -44,17 +44,22 @@
 
 #include <string.h>
 #include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomevfs/gnome-vfs-dns-sd.h>
 #include <gtk/gtkmessagedialog.h>
 #include <gtk/gtkdialog.h>
 
+#ifdef ENABLE_ZEROCONF
+#include <avahi-common/error.h>
+#include <avahi-gobject/ga-service-browser.h>
+#include <avahi-gobject/ga-service-resolver.h>
+#include <avahi-gobject/ga-client.h>
+#include <avahi-gobject/ga-enums.h>
+#endif /* ENABLE_ZEROCONF */
+
 #define EPHY_BOOKMARKS_XML_ROOT    "ephy_bookmarks"
 #define EPHY_BOOKMARKS_XML_VERSION "1.03"
 #define BOOKMARKS_SAVE_DELAY 3 /* seconds */
 #define MAX_FAVORITES_NUM 10
 #define UPDATE_URI_DATA_KEY "updated-uri"
-#define SD_RESOLVE_TIMEOUT 0 /* ms; 0 means no timeout */
 
 #define EPHY_BOOKMARKS_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_BOOKMARKS, EphyBookmarksPrivate))
 
@@ -85,7 +90,8 @@ struct _EphyBookmarksPrivate
 #ifdef ENABLE_ZEROCONF
 	/* Local sites */
 	EphyNode *local;
-	GnomeVFSDNSSDBrowseHandle *browse_handles[G_N_ELEMENTS (zeroconf_protos)];
+	GaClient *ga_client;
+	GaServiceBrowser *browse_handles[G_N_ELEMENTS (zeroconf_protos)];
 	GHashTable *resolve_handles;
 #endif
 };
@@ -129,40 +135,12 @@ static void ephy_bookmarks_class_init	(EphyBookmarksClass *klass);
 static void ephy_bookmarks_init		(EphyBookmarks *tab);
 static void ephy_bookmarks_finalize	(GObject *object);
 static char *impl_resolve_address	(EphyBookmarks*, const char*, const char*);
+static void ephy_local_bookmarks_start_client (EphyBookmarks *bookmarks);
 
-static GObjectClass *parent_class = NULL;
-
-GType
-ephy_bookmarks_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (EphyBookmarksClass),
-			NULL, /* base_init */
-			NULL, /* base_finalize */
-			(GClassInitFunc) ephy_bookmarks_class_init,
-			NULL,
-			NULL, /* class_data */
-			sizeof (EphyBookmarks),
-			0, /* n_preallocs */
-			(GInstanceInitFunc) ephy_bookmarks_init
-		};
-		volatile GType flags_type; /* work around gcc's optimiser */
-      
-		/* make sure the flags type is known */
-	      	flags_type = EPHY_TYPE_BOOKMARK_PROPERTY;
-
-		type = g_type_register_static (G_TYPE_OBJECT,
-					       "EphyBookmarks",
-					       &our_info, 0);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE_WITH_CODE (EphyBookmarks, ephy_bookmarks, G_TYPE_OBJECT,
+			 volatile GType flags_type; /* work around gcc's optimiser */
+			 /* make sure the flags type is known */
+		      	 flags_type = EPHY_TYPE_BOOKMARK_PROPERTY;)
 
 static void
 ephy_bookmarks_init_defaults (EphyBookmarks *eb)
@@ -188,8 +166,6 @@ ephy_bookmarks_class_init (EphyBookmarksClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = ephy_bookmarks_finalize;
 
 	klass->resolve_address = impl_resolve_address;
@@ -782,248 +758,423 @@ backup_file (const char *original_filename, const char *extension)
 
 #ifdef ENABLE_ZEROCONF
 
-static char *
-get_id_for_service (const GnomeVFSDNSSDService *service)
+/* C&P adapted from gnome-vfs-dns-sd.c */
+static GHashTable *
+decode_txt_record (AvahiStringList *input_text)
 {
-	/* FIXME: limit length! */
-	return g_strdup_printf ("%s\1%s\1%s",
-				service->type,
-				service->domain,
-				service->name);
-}
+	GHashTable *hash;
+	int i;
+	int len;
+	char *key, *value, *end;
+	char *key_dup, *value_dup;
+	char *raw_txt;
+	size_t raw_txt_len;
 
-static EphyNode *
-get_node_for_service (EphyBookmarks *bookmarks,
-		      const GnomeVFSDNSSDService *service)
-{
-	EphyBookmarksPrivate *priv = bookmarks->priv;
-	EphyNode *kid, *node = NULL;
-	GPtrArray *children;
-	char *search_id;
-	const char *id;
-	guint i;
+	if (!input_text)
+		return NULL;
 
-	search_id = get_id_for_service (service);
+	raw_txt_len = avahi_string_list_serialize (input_text, NULL, 0);
+	raw_txt = g_malloc (raw_txt_len);
+	raw_txt_len = avahi_string_list_serialize (input_text, raw_txt, raw_txt_len);
 
-	children = ephy_node_get_children (priv->local);
-	for (i = 0; i < children->len; i++)
-	{
-		kid = g_ptr_array_index (children, i);
+	if (raw_txt == NULL)
+		return NULL;
+	
+	hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
-		id = ephy_node_get_property_string (kid,
-						    EPHY_NODE_BMK_PROP_SERVICE_ID);
-
-		if (g_str_equal (id, search_id))
-		{
-			node = kid;
+	i = 0;
+	while (i < raw_txt_len) {
+		len = raw_txt[i++];
+		
+		if (i + len > raw_txt_len) {
 			break;
 		}
+		
+		if (len == 0) {
+			continue;
+		}
+		
+		key = &raw_txt[i];
+		end = &raw_txt[i + len];
+		i += len;
+
+		if (*key == '=') {
+			/* 6.4 - silently ignore keys starting with = */
+			continue;
+		}
+		
+		value = memchr (key, '=', len);
+		if (value) {
+			key_dup = g_strndup (key, value - key);
+			value++; /* Skip '=' */
+			value_dup = g_strndup (value, end - value);
+		} else {
+			key_dup = g_strndup (key, len);
+			value_dup = NULL;
+		}
+		if (!g_hash_table_lookup_extended (hash,
+						   key_dup,
+						   NULL, NULL)) {
+			g_hash_table_insert (hash,
+					     key_dup,
+					     value_dup);
+		} else {
+			g_free (key_dup);
+			g_free (value_dup);
+		}
 	}
 
-	g_free (search_id);
+	return hash;
+}
 
-	return node;
+/* End of copied code */
+
+static char *
+get_id_for_response (const char *type,
+		     const char *domain,
+		     const char *name)
+{
+	/* FIXME: limit length! */
+	return g_strdup_printf ("%s\1%s\1%s",
+				type,
+				domain,
+				name);
 }
 
 typedef struct
 {
 	EphyBookmarks *bookmarks;
+	GaServiceResolver *resolver;
 	EphyNode *node;
-	guint new_node : 1;
+	char *name;
+	char *type;
+	char *domain;
 } ResolveData;
 
 static void
-resolve_cb (GnomeVFSDNSSDResolveHandle *handle,
-	    GnomeVFSResult result,
-	    const GnomeVFSDNSSDService *service,
-	    const char *host,
-	    int port,
-	    /* const */ GHashTable *text,
-	    int text_raw_len,
-	    const char *text_raw,
-	    ResolveData *data)
+resolver_found_cb (GaServiceResolver *resolver,
+		   int interface,
+		   GaProtocol protocol,
+		   const char *name,
+		   const char *type,
+		   const char *domain,
+		   const char *host_name,
+		   const AvahiAddress *address,
+		   guint16 port,
+		   AvahiStringList *txt,
+		   glong flags,
+		   ResolveData *data)
 {
 	EphyBookmarks *bookmarks = data->bookmarks;
 	EphyBookmarksPrivate *priv = bookmarks->priv;
-	EphyNode *node = data->node;
 	GValue value = { 0, };
 	const char *path = NULL;
+	char host[128];
+	GHashTable *text_table;
 	char *url;
 	gboolean was_immutable;
+	gboolean is_new_node = FALSE;
 	guint i;
 
+	LOG ("resolver_found_cb resolver %p\n", resolver);
+
 	was_immutable = ephy_node_db_is_immutable (priv->db);
 	ephy_node_db_set_immutable (priv->db, FALSE);
 
-	g_hash_table_steal (priv->resolve_handles, node);
-
-	/* Error, don't add the service */
-	if (result != GNOME_VFS_OK)
-	{
-		ephy_node_unref (node);
-
-		ephy_node_db_set_immutable (priv->db, was_immutable);
-
-		return;
-	}
-
 	/* Find the protocol */
 	for (i = 0; i < G_N_ELEMENTS (zeroconf_protos); ++i)
 	{
 		char proto[20];
 
 		g_snprintf (proto, sizeof (proto), "_%s._tcp", zeroconf_protos[i]);
-		if (strcmp (service->type, proto) == 0) break;
+		if (strcmp (type, proto) == 0) break;
 	}
 	if (i == G_N_ELEMENTS (zeroconf_protos)) return;
-
-	if (text != NULL)
+	
+	if (address == NULL)
 	{
-		path = g_hash_table_lookup (text, "path");
+		g_warning ("Zeroconf failed to resolve host %s", name);
+		return;
+	}
+
+	text_table = decode_txt_record (txt);
+
+	if (text_table != NULL)
+	{
+		path = g_hash_table_lookup (text_table, "path");
 	}
 	if (path == NULL || path[0] == '\0')
 	{
 		path = "/";
 	}
+	
+	avahi_address_snprint (host, sizeof (host), address);
 
 	LOG ("0conf RESOLVED type=%s domain=%s name=%s => proto=%s host=%s port=%d path=%s\n",
-	    service->type, service->domain, service->name,
-	    zeroconf_protos[i], host, port, path);
+	     type, domain, name,
+	     zeroconf_protos[i], host, port, path);
+
+	was_immutable = ephy_node_db_is_immutable (priv->db);
+	ephy_node_db_set_immutable (priv->db, FALSE);
+
+	if (data->node == NULL)
+	{
+		is_new_node = TRUE;
+
+		data->node = ephy_node_new (priv->db);
+		g_assert (data->node != NULL);
+
+		/* don't allow dragging this node */
+		ephy_node_set_is_drag_source (data->node, FALSE);
+
+		g_value_init (&value, G_TYPE_STRING);
+		g_value_take_string (&value,
+				     get_id_for_response (data->type, 
+							  data->domain,
+							  data->name));
+		ephy_node_set_property (data->node, EPHY_NODE_BMK_PROP_SERVICE_ID, &value);
+		g_value_unset (&value);
+
+		/* FIXME: limit length! */
+		ephy_node_set_property_string (data->node,
+					       EPHY_NODE_BMK_PROP_TITLE,
+					       name);
+
+		ephy_node_set_property_boolean (data->node,
+						EPHY_NODE_BMK_PROP_IMMUTABLE,
+						TRUE);
+	}
 
 	/* FIXME: limit length! */
 	url = g_strdup_printf ("%s://%s:%d%s", zeroconf_protos[i], host, port, path);
 
 	g_value_init (&value, G_TYPE_STRING);
 	g_value_take_string (&value, url);
-	ephy_node_set_property (node, EPHY_NODE_BMK_PROP_LOCATION, &value);
+	ephy_node_set_property (data->node, EPHY_NODE_BMK_PROP_LOCATION, &value);
 	g_value_unset (&value);
 
-	if (data->new_node)
+	if (is_new_node)
 	{
-		ephy_node_add_child (priv->bookmarks, node);
-		ephy_node_add_child (priv->local, node);
+		ephy_node_add_child (priv->bookmarks, data->node);
+		ephy_node_add_child (priv->local, data->node);
 	}
 	
 	ephy_node_db_set_immutable (priv->db, was_immutable);
+
+	g_hash_table_unref (text_table);
 }
 
 static void
-browse_cb (GnomeVFSDNSSDBrowseHandle *handle,
-	   GnomeVFSDNSSDServiceStatus status,
-	   const GnomeVFSDNSSDService *service,
-	   EphyBookmarks *bookmarks)
+resolver_failure_cb (GaServiceResolver *resolver,
+		     GError *error,
+		     ResolveData *data)
 {
-	EphyBookmarksPrivate *priv = bookmarks->priv;
-	GnomeVFSDNSSDResolveHandle *reshandle = NULL;
-	ResolveData *data;
-	EphyNode *node;
-	GValue value = { 0, };
-	gboolean new_node = FALSE;
+	LOG ("resolver_failure_cb resolver %p: %s\n", resolver, error?error->message:"(null)");
 
-	if (service == NULL) return;
-
-	node = get_node_for_service (bookmarks, service);
-
-	if (status == GNOME_VFS_DNS_SD_SERVICE_REMOVED)
-	{
-		if (node != NULL)
-		{
-			g_hash_table_remove (priv->resolve_handles, node);
-			ephy_node_unref (node);
-		}
-
-		return;
-	}
-
-	/* status == GNOME_VFS_DNS_SD_SERVICE_ADDED */
-
-	LOG ("0conf ADD: type=%s domain=%s name=%s\n",
-	      service->type, service->domain, service->name);
-
-	if (node != NULL &&
-	    g_hash_table_lookup (priv->resolve_handles, node) != NULL) return;
-
-	if (node == NULL)
-	{
+	/* Remove the node, if present */
+	if (data->node != NULL)
+	{	
+		EphyBookmarks *bookmarks = data->bookmarks;
+		EphyBookmarksPrivate *priv = bookmarks->priv;
 		gboolean was_immutable;
 
 		was_immutable = ephy_node_db_is_immutable (priv->db);
-		ephy_node_db_set_immutable (priv->db, FALSE);
-
-		node = ephy_node_new (priv->db);
-		g_assert (node != NULL);
-
-		new_node = TRUE;
-
-		/* don't allow dragging this node */
-		ephy_node_set_is_drag_source (node, FALSE);
-
-		g_value_init (&value, G_TYPE_STRING);
-		g_value_take_string (&value, get_id_for_service (service));
-		ephy_node_set_property (node, EPHY_NODE_BMK_PROP_SERVICE_ID, &value);
-		g_value_unset (&value);
-
-		/* FIXME: limit length! */
-		ephy_node_set_property_string (node,
-					       EPHY_NODE_BMK_PROP_TITLE,
-					       service->name);
-
-		ephy_node_set_property_boolean (node,
-						EPHY_NODE_BMK_PROP_IMMUTABLE,
-						TRUE);
-
+		ephy_node_db_set_immutable (priv->db, FALSE);	
+		ephy_node_unref (data->node);
+		data->node = NULL;
 		ephy_node_db_set_immutable (priv->db, was_immutable);
 	}
+}
 
-	data = g_new (ResolveData, 1);
-	data->bookmarks = bookmarks;
-	data->node = node;
-	data->new_node = new_node;
+static void
+resolve_data_free (ResolveData* data)
+{
+	if (data->resolver)
+		g_object_unref (data->resolver);
 
-	if (gnome_vfs_dns_sd_resolve (&reshandle,
-				      service->name, service->type, service->domain,
-				      SD_RESOLVE_TIMEOUT,
-				      (GnomeVFSDNSSDResolveCallback) resolve_cb,
-				      data,
-				      (GDestroyNotify) g_free) != GNOME_VFS_OK)
+	g_free (data->type);
+	g_free (data->name);
+	g_free (data->domain);
+	g_slice_free (ResolveData, data);
+}
+
+static void
+browser_new_service_cb (GaServiceBrowser *browser,
+			int interface,
+			GaProtocol protocol,
+			const char *name,
+			const char *type,
+			const char *domain,
+			glong flags,
+			EphyBookmarks *bookmarks)
+{
+	EphyBookmarksPrivate *priv = bookmarks->priv;
+	ResolveData *data;
+	char *node_id;
+	GError *error = NULL;
+	
+	node_id = get_id_for_response (type, domain, name);
+	
+	LOG ("0conf ADD: type=%s domain=%s name=%s\n",
+	     type, domain, name);
+	
+	if (g_hash_table_lookup (priv->resolve_handles, node_id) != NULL)
 	{
-		ephy_node_unref (node);
-		g_free (data);
+		g_free (node_id);
+		return;
+	}
 
+	data = g_slice_new0 (ResolveData);
+	data->bookmarks = bookmarks;
+	data->node = NULL;
+	data->type = g_strdup (type);
+	data->name = g_strdup (name);
+	data->domain = g_strdup (domain);
+	
+	data->resolver = ga_service_resolver_new (AVAHI_IF_UNSPEC,
+						  AVAHI_PROTO_UNSPEC,
+						  name, type, domain,
+						  AVAHI_PROTO_UNSPEC,
+						  GA_LOOKUP_USE_MULTICAST);
+	g_signal_connect (data->resolver, "found",
+			  G_CALLBACK (resolver_found_cb), data);
+	g_signal_connect (data->resolver, "failure",
+			  G_CALLBACK (resolver_failure_cb), data);
+	if (!ga_service_resolver_attach (data->resolver,
+					 priv->ga_client,
+					 &error))
+	{
+		g_warning ("Unable to resolve Zeroconf service %s: %s", name, error ? error->message : "(null)");
+		g_clear_error (&error);
+		resolve_data_free (data);
+		g_free (node_id);
 		return;
 	}
 
 	g_hash_table_insert (priv->resolve_handles,
-			     node, reshandle);
+			     node_id /* transfer ownership */, data);
+}
+
+static void
+browser_removed_service_cb (GaServiceBrowser *browser,
+			    int interface,
+			    GaProtocol protocol,
+			    const char *name,
+			    const char *type,
+			    const char *domain,
+			    glong flags,
+			    EphyBookmarks *bookmarks)
+{
+	EphyBookmarksPrivate *priv = bookmarks->priv;
+	char *node_id;
+	ResolveData *data;
+
+	node_id = get_id_for_response (type, domain, name);
+	data = g_hash_table_lookup (priv->resolve_handles, node_id);
+	/* shouldn't really happen, but let's play safe */
+	if (!data)
+	{
+		g_free (node_id);
+		return;
+	}
+
+	if (data->node != NULL)
+	{	
+		gboolean was_immutable;
+
+		was_immutable = ephy_node_db_is_immutable (priv->db);
+		ephy_node_db_set_immutable (priv->db, FALSE);	
+		ephy_node_unref (data->node);
+		data->node = NULL;
+		ephy_node_db_set_immutable (priv->db, was_immutable);
+	}
+
+	g_hash_table_remove (priv->resolve_handles, node_id);
+	g_free (node_id);
+}
+
+static void
+start_browsing (GaClient *ga_client,
+		EphyBookmarks *bookmarks)
+{
+	EphyBookmarksPrivate *priv = bookmarks->priv;
+	guint i;
+
+	for (i = 0; i < G_N_ELEMENTS (zeroconf_protos); ++i)
+	{
+		GaServiceBrowser *browser = NULL;
+		char proto[20];
+
+		g_snprintf (proto, sizeof (proto), "_%s._tcp", zeroconf_protos[i]);
+
+		browser = ga_service_browser_new (proto);
+		g_signal_connect (browser, "new-service",
+				  G_CALLBACK (browser_new_service_cb), bookmarks);
+		g_signal_connect (browser, "removed-service",
+				  G_CALLBACK (browser_removed_service_cb), bookmarks);
+		if (!ga_service_browser_attach (browser,
+						ga_client,
+						NULL))
+		{
+			g_warning ("Unable to start Zeroconf subsystem");
+			g_object_unref (browser);
+			return;
+		}
+
+		priv->browse_handles[i] = browser;
+	}
+}
+
+static void
+ga_client_state_changed_cb (GaClient *ga_client,
+			    GaClientState state,
+			    EphyBookmarks *bookmarks)
+{
+	if (state == GA_CLIENT_STATE_FAILURE)
+	{
+		if (avahi_client_errno (ga_client->avahi_client) == AVAHI_ERR_DISCONNECTED)
+		{
+                        EphyBookmarksPrivate *priv = bookmarks->priv;
+
+                        g_object_unref (priv->ga_client);
+                        priv->ga_client = NULL;
+
+                        ephy_local_bookmarks_start_client (bookmarks);
+		}
+	}
+	if (state == GA_CLIENT_STATE_S_RUNNING)
+	{
+		start_browsing (ga_client, bookmarks);
+	}
+}
+
+static void
+ephy_local_bookmarks_start_client (EphyBookmarks *bookmarks)
+{
+	EphyBookmarksPrivate *priv = bookmarks->priv;
+	GaClient *ga_client;
+
+	ga_client = ga_client_new (GA_CLIENT_FLAG_NO_FAIL);
+	g_signal_connect (ga_client, "state-changed",
+			  G_CALLBACK (ga_client_state_changed_cb),
+			  bookmarks);
+	if (!ga_client_start (ga_client, NULL))
+	{
+		g_warning ("Unable to start Zeroconf subsystem");
+                g_object_unref (ga_client);
+		return;
+	}
+	priv->ga_client = ga_client;
 }
 
 static void
 ephy_local_bookmarks_init (EphyBookmarks *bookmarks)
 {
 	EphyBookmarksPrivate *priv = bookmarks->priv;
-	guint i;
-
-	priv->resolve_handles =
-		g_hash_table_new_full (g_direct_hash, g_direct_equal,
-				       NULL,
-				       (GDestroyNotify) gnome_vfs_dns_sd_cancel_resolve);
-
-	for (i = 0; i < G_N_ELEMENTS (zeroconf_protos); ++i)
-	{
-		GnomeVFSDNSSDBrowseHandle *handle = NULL;
-		char proto[20];
-
-		g_snprintf (proto, sizeof (proto), "_%s._tcp", zeroconf_protos[i]);
-
-		if (gnome_vfs_dns_sd_browse (&handle,
-		    			     "local", proto,
-					     (GnomeVFSDNSSDBrowseCallback) browse_cb,
-					     bookmarks,
-					     NULL) == GNOME_VFS_OK)
-		{
-			priv->browse_handles[i] = handle;
-		}
-	}
+	priv->resolve_handles =	g_hash_table_new_full (g_str_hash, g_str_equal,
+						       g_free,
+						       (GDestroyNotify) resolve_data_free);
+        ephy_local_bookmarks_start_client (bookmarks);
 }
 
 static void
@@ -1036,7 +1187,7 @@ ephy_local_bookmarks_stop (EphyBookmarks *bookmarks)
 	{
 		if (priv->browse_handles[i] != NULL)
 		{
-			gnome_vfs_dns_sd_stop_browse (priv->browse_handles[i]);
+			g_object_unref (priv->browse_handles[i]);
 			priv->browse_handles[i] = NULL;
 		}
 	}
@@ -1049,7 +1200,14 @@ ephy_local_bookmarks_stop (EphyBookmarks *bookmarks)
 
 	if (priv->local != NULL)
 	{
-		ephy_node_remove_child (priv->keywords, priv->local);
+		ephy_node_unref (priv->local);
+		priv->local = NULL;
+	}
+	
+	if (priv->ga_client != NULL)
+	{
+		g_object_unref (priv->ga_client);
+		priv->ga_client = NULL;
 	}
 }
 
@@ -1252,7 +1410,7 @@ ephy_bookmarks_finalize (GObject *object)
 
 	LOG ("Bookmarks finalized");
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (ephy_bookmarks_parent_class)->finalize (object);
 }
 
 EphyBookmarks *
@@ -1569,7 +1727,7 @@ impl_resolve_address (EphyBookmarks *eb,
 			}
 			else
 			{
-				escaped_arg = gnome_vfs_escape_string (arg);
+				escaped_arg = g_uri_escape_string (arg, NULL, TRUE);
 				g_string_append (result, escaped_arg);
 				g_free (escaped_arg);
 				g_free (arg);
@@ -1579,7 +1737,7 @@ impl_resolve_address (EphyBookmarks *eb,
 		}
 		else
 		{
-			arg = gnome_vfs_escape_string (content);
+			arg = g_uri_escape_string (content, NULL, TRUE);
 			g_string_append (result, arg);
 			g_free (arg);
 		}
