@@ -14,9 +14,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  $Id: ephy-find-toolbar.c 6940 2007-03-05 19:17:20Z chpe $
+ *  $Id$
  */
 
 #include "config.h"
@@ -28,7 +28,9 @@
 
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
+#include <gtk/gtk.h>
 #include <gtk/gtkarrow.h>
+#include <gtk/gtkcheckbutton.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkentry.h>
@@ -54,7 +56,10 @@ struct _EphyFindToolbarPrivate
 	GtkToolItem *sep;
 	GtkToolItem *status_item;
 	GtkWidget *status_label;
+	GtkWidget *case_sensitive;
 	gulong set_focus_handler;
+	guint source_id;
+	guint find_again_source_id;
 	guint preedit_changed : 1;
 	guint prevent_activate : 1;
 	guint activated : 1;
@@ -76,9 +81,7 @@ enum
 	LAST_SIGNAL
 };
 
-static guint signals[LAST_SIGNAL] = { 0 };
-
-static GObjectClass *parent_class = NULL;
+static guint signals[LAST_SIGNAL];
 
 /* private functions */
 
@@ -101,12 +104,30 @@ get_find (EphyFindToolbar *toolbar)
 	return priv->find;
 }
 
+static gboolean
+set_status_notfound_cb (EphyFindToolbar *toolbar)
+{
+	EphyFindToolbarPrivate *priv;
+	PangoFontDescription *pango_desc = NULL;
+	
+	priv = toolbar->priv;
+	
+	pango_desc = pango_font_description_new ();
+	gtk_widget_modify_font (priv->status_label, pango_desc);
+	pango_font_description_free (pango_desc);
+	
+	priv->source_id = 0;
+	
+	return FALSE;
+}
+
 static void
 set_status (EphyFindToolbar *toolbar,
 	    EphyEmbedFindResult result)
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 	char *text = NULL;
+	PangoFontDescription *pango_desc = NULL;
 
 	switch (result)
 	{
@@ -114,7 +135,17 @@ set_status (EphyFindToolbar *toolbar,
 			text = NULL;
 			break;
 		case EPHY_EMBED_FIND_NOTFOUND:
-			text = _("Not found");
+			{
+				text = _("Not found");
+
+				pango_desc = pango_font_description_new ();
+				pango_font_description_set_weight (pango_desc, PANGO_WEIGHT_BOLD);
+				gtk_widget_modify_font (priv->status_label, pango_desc);
+				pango_font_description_free (pango_desc);
+				
+				gtk_widget_error_bell (GTK_WIDGET (priv->window));
+				priv->source_id = g_timeout_add (500, (GSourceFunc) set_status_notfound_cb, toolbar);
+			}
 			break;
 		case EPHY_EMBED_FIND_FOUNDWRAPPED:
 			text = _("Wrapped");
@@ -192,18 +223,11 @@ entry_changed_cb (GtkEntry *entry,
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 	const char *text;
-	char *lowercase;
 	EphyEmbedFindResult result;
 	gboolean case_sensitive;
 
 	text = gtk_entry_get_text (GTK_ENTRY (priv->entry));
-
-	/* Search case-sensitively iff the string includes 
-	 * non-lowercase character.
-	 */
-	lowercase = g_utf8_strdown (text, -1);
-	case_sensitive = g_utf8_collate (text, lowercase) != 0;
-	g_free (lowercase);
+	case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->case_sensitive));
 
 	ephy_embed_find_set_properties (get_find (toolbar), text, case_sensitive);
 	result = ephy_embed_find_find (get_find (toolbar), text, priv->links_only);
@@ -257,6 +281,14 @@ entry_key_press_event_cb (GtkEntry *entry,
 	{
 		handled = ephy_embed_find_activate_link (get_find (toolbar), event->state);
 	}
+	else if ((event->state & mask) == GDK_SHIFT_MASK &&
+		 (event->keyval == GDK_Return ||
+		  event->keyval == GDK_KP_Enter ||
+		  event->keyval == GDK_ISO_Enter))
+	{
+		handled = TRUE;
+		g_signal_emit (toolbar, signals[PREVIOUS], 0);
+	}
 
 	return handled;
 }
@@ -277,6 +309,71 @@ entry_activate_cb (GtkWidget *entry,
 	}
 }
 
+static void
+case_sensitive_menu_toggled_cb (GtkWidget *check,
+				EphyFindToolbar *toolbar)
+{
+	EphyFindToolbarPrivate *priv = toolbar->priv;
+	gboolean case_sensitive;
+
+	case_sensitive = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (check));
+
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->case_sensitive),
+				      case_sensitive);
+}
+
+static void
+case_sensitive_toggled_cb (GtkWidget *check,
+			   EphyFindToolbar *toolbar)
+{
+	EphyFindToolbarPrivate *priv = toolbar->priv;
+	const char *text;
+	gboolean case_sensitive;
+	GtkWidget *proxy;
+
+	text = gtk_entry_get_text (GTK_ENTRY (priv->entry));
+	case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check));
+
+	proxy = gtk_tool_item_get_proxy_menu_item (GTK_TOOL_ITEM (gtk_widget_get_parent (check)),
+						   "menu-proxy");
+
+	if (proxy != NULL)
+	{
+		g_signal_handlers_block_by_func
+			(proxy, G_CALLBACK (case_sensitive_menu_toggled_cb), toolbar);
+
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (proxy),
+						case_sensitive);
+		g_signal_handlers_unblock_by_func
+			(proxy, G_CALLBACK (case_sensitive_menu_toggled_cb), toolbar);
+	}
+
+	ephy_embed_find_set_properties (get_find (toolbar), text, case_sensitive);
+}
+
+static gboolean
+toolitem_create_menu_proxy_cb (GtkToolItem *toolitem,
+			       EphyFindToolbar *toolbar)
+{
+	EphyFindToolbarPrivate *priv = toolbar->priv;
+	GtkWidget *checkbox_menu;
+	gboolean case_sensitive;
+
+	/* Create a menu item, and sync it */
+	checkbox_menu = gtk_check_menu_item_new_with_mnemonic (_("_Case sensitive"));
+	case_sensitive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->case_sensitive));
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (checkbox_menu),
+					case_sensitive);
+
+	g_signal_connect (checkbox_menu, "toggled",
+			  G_CALLBACK (case_sensitive_menu_toggled_cb), toolbar);
+
+	gtk_tool_item_set_proxy_menu_item (toolitem, "menu-proxy",
+					   checkbox_menu);
+
+	return TRUE;
+}
+
 static void
 set_focus_cb (EphyWindow *window,
 	      GtkWidget *widget,
@@ -338,6 +435,7 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 	GtkToolbar *gtoolbar;
 	GtkToolItem *item;
 	GtkWidget *alignment, *arrow, *box;
+	GtkWidget *checkbox;
 
 	priv = toolbar->priv = EPHY_FIND_TOOLBAR_GET_PRIVATE (toolbar);
 	gtoolbar = GTK_TOOLBAR (toolbar);
@@ -369,9 +467,8 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 	arrow = gtk_arrow_new (GTK_ARROW_LEFT, GTK_SHADOW_NONE);
 	priv->prev = gtk_tool_button_new (arrow, _("Find Previous"));
 	gtk_tool_item_set_is_important (priv->prev, TRUE);
-	gtk_tool_item_set_tooltip (priv->prev, gtoolbar->tooltips,
-				   _("Find previous occurrence of the search string"),
-				   NULL);
+	gtk_tool_item_set_tooltip_text (priv->prev,
+					_("Find previous occurrence of the search string"));
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), priv->prev, -1);
 	gtk_widget_show_all (GTK_WIDGET (priv->prev));
 
@@ -379,12 +476,23 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 	arrow = gtk_arrow_new (GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
 	priv->next = gtk_tool_button_new (arrow, _("Find Next"));
 	gtk_tool_item_set_is_important (priv->next, TRUE);
-	gtk_tool_item_set_tooltip (priv->next, gtoolbar->tooltips,
-				   _("Find next occurrence of the search string"),
-				   NULL);
+	gtk_tool_item_set_tooltip_text (priv->next,
+					_("Find next occurrence of the search string"));
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), priv->next, -1);
 	gtk_widget_show_all (GTK_WIDGET (priv->next));
 
+	/* Case sensitivity */
+	checkbox = gtk_check_button_new_with_mnemonic (_("_Case sensitive"));
+	priv->case_sensitive = checkbox;
+	item = gtk_tool_item_new ();
+	gtk_container_add (GTK_CONTAINER (item), checkbox);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
+	gtk_widget_show_all (GTK_WIDGET (item));
+
+	/* Populate the overflow menu */
+	g_signal_connect (item, "create-menu-proxy",
+			  G_CALLBACK (toolitem_create_menu_proxy_cb), toolbar);
+
 	priv->sep = gtk_separator_tool_item_new ();
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), priv->sep, -1);
 	
@@ -408,8 +516,12 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 				  G_CALLBACK (find_next_cb), toolbar);
 	g_signal_connect_swapped (priv->prev, "clicked",
 				  G_CALLBACK (find_prev_cb), toolbar);
+	g_signal_connect (priv->case_sensitive, "toggled",
+			  G_CALLBACK (case_sensitive_toggled_cb), toolbar);
 }
 
+G_DEFINE_TYPE (EphyFindToolbar, ephy_find_toolbar, GTK_TYPE_TOOLBAR)
+
 static void
 ephy_find_toolbar_dispose (GObject *object)
 {
@@ -421,8 +533,20 @@ ephy_find_toolbar_dispose (GObject *object)
 		g_object_unref (priv->find);
 		priv->find = NULL;
 	}
+	
+	if (priv->source_id != 0)
+	{
+		g_source_remove (priv->source_id);
+		priv->source_id = 0;
+	}
 
-	parent_class->dispose (object);
+	if (priv->find_again_source_id != 0)
+	{
+		g_source_remove (priv->find_again_source_id);
+		priv->find_again_source_id = 0;
+	}
+
+	G_OBJECT_CLASS (ephy_find_toolbar_parent_class)->dispose (object);
 }
 
 static void
@@ -457,8 +581,6 @@ ephy_find_toolbar_class_init (EphyFindToolbarClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
         GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-        parent_class = g_type_class_peek_parent (klass);
-
 	object_class->dispose = ephy_find_toolbar_dispose;
 	object_class->get_property = ephy_find_toolbar_get_property;
 	object_class->set_property = ephy_find_toolbar_set_property;
@@ -510,34 +632,6 @@ ephy_find_toolbar_class_init (EphyFindToolbarClass *klass)
 
 /* public functions */
 
-GType
-ephy_find_toolbar_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (EphyFindToolbarClass),
-			NULL, /* base_init */
-			NULL, /* base_finalize */
-			(GClassInitFunc) ephy_find_toolbar_class_init,
-			NULL,
-			NULL, /* class_data */
-			sizeof (EphyFindToolbar),
-			0, /* n_preallocs */
-			(GInstanceInitFunc) ephy_find_toolbar_init
-		};
-	
-		type = g_type_register_static (GTK_TYPE_TOOLBAR,
-					       "EphyFindToolbar",
-					       &our_info, 0);
-	}
-
-	return type;
-}
-
 EphyFindToolbar *
 ephy_find_toolbar_new (EphyWindow *window)
 {
@@ -586,21 +680,61 @@ ephy_find_toolbar_set_embed (EphyFindToolbar *toolbar,
 	}
 }
 
+typedef struct
+{
+	EphyFindToolbar *toolbar;
+	gboolean next;
+} FindAgainCBStruct;
+
+static void
+find_again_data_destroy_cb (FindAgainCBStruct *data)
+{
+	g_slice_free (FindAgainCBStruct, data);
+}
+	
+
+static gboolean
+find_again_cb (FindAgainCBStruct *data)
+{
+	EphyEmbedFindResult result;
+	EphyFindToolbarPrivate *priv = data->toolbar->priv;
+
+	result = ephy_embed_find_find_again (get_find (data->toolbar), data->next,
+					     priv->links_only);
+	set_status (data->toolbar, result);
+    
+	priv->find_again_source_id = 0;
+    
+	return FALSE;
+}
+
 void
 ephy_find_toolbar_find_next (EphyFindToolbar *toolbar)
 {
 	GtkWidget *widget = GTK_WIDGET (toolbar);
 	EphyFindToolbarPrivate *priv = toolbar->priv;
-	EphyEmbedFindResult result;
-
-	result = ephy_embed_find_find_again (get_find (toolbar), TRUE,
-					     priv->links_only);
-	set_status (toolbar, result);
+	FindAgainCBStruct *data;
 
 	if (!GTK_WIDGET_VISIBLE (widget)) {
 		gtk_widget_show (widget);
 		gtk_widget_grab_focus (widget);
 	}
+
+	/* We need to do this here (and in find_previous) to give time to the embed
+	 * to sync with the size change due to the toolbar being shown, otherwise
+	 * the toolbar can obscure the result. See GNOME bug #415074.
+	 */
+    
+	if (priv->find_again_source_id != 0) return;
+
+    	data = g_slice_new0 (FindAgainCBStruct);
+	data->toolbar = toolbar;
+	data->next = TRUE;
+    
+	priv->find_again_source_id = g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+						      (GSourceFunc) find_again_cb,
+						      data,
+						      (GDestroyNotify) find_again_data_destroy_cb);
 }
 
 void
@@ -608,16 +742,23 @@ ephy_find_toolbar_find_previous (EphyFindToolbar *toolbar)
 {
 	GtkWidget *widget = GTK_WIDGET (toolbar);
 	EphyFindToolbarPrivate *priv = toolbar->priv;
-	EphyEmbedFindResult result;
-
-	result = ephy_embed_find_find_again (get_find (toolbar), FALSE,
-					     priv->links_only);
-	set_status (toolbar, result);
+	FindAgainCBStruct *data;
 
 	if (!GTK_WIDGET_VISIBLE (widget)) {
 		gtk_widget_show (widget);
 		gtk_widget_grab_focus (widget);
 	}
+    
+	if (priv->find_again_source_id != 0) return;
+
+	data = g_slice_new0 (FindAgainCBStruct);
+	data->toolbar = toolbar;
+	data->next = FALSE;
+    
+	priv->find_again_source_id = g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+						      (GSourceFunc) find_again_cb,
+						      data,
+						      (GDestroyNotify) find_again_data_destroy_cb);
 }
 
 void
@@ -626,7 +767,6 @@ ephy_find_toolbar_open (EphyFindToolbar *toolbar,
 			gboolean typing_ahead)
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
-	gboolean clear_search = typing_ahead;
 
 	g_return_if_fail (priv->embed != NULL);
 
@@ -635,14 +775,7 @@ ephy_find_toolbar_open (EphyFindToolbar *toolbar,
 
 	clear_status (toolbar);
 
-	if (clear_search)
-	{
-		gtk_entry_set_text (GTK_ENTRY (priv->entry), "");
-	}
-	else
-	{
-		gtk_editable_select_region (GTK_EDITABLE (priv->entry), 0, -1);
-	}
+	gtk_editable_select_region (GTK_EDITABLE (priv->entry), 0, -1);
 
 	gtk_widget_show (GTK_WIDGET (toolbar));
 	gtk_widget_grab_focus (GTK_WIDGET (toolbar));
