@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-history-window.c 7719 2007-11-18 23:53:16Z cosimoc $
+ *  $Id$
  */
 
 #include "config.h"
@@ -164,8 +164,6 @@ enum
 #define TIME_LAST_TWO_DAYS_STRING "last_two_days"
 #define TIME_LAST_THREE_DAYS_STRING "last_three_days"
 
-static GObjectClass *parent_class = NULL;
-
 static const GtkActionEntry ephy_history_ui_entries [] = {
 	/* Toplevel */
 	{ "File", NULL, N_("_File") },
@@ -590,33 +588,7 @@ cmd_view_columns (GtkAction *action,
 	g_slist_free (svalues);
 }
 
-GType
-ephy_history_window_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (EphyHistoryWindowClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) ephy_history_window_class_init,
-			NULL,
-			NULL,
-			sizeof (EphyHistoryWindow),
-			0,
-			(GInstanceInitFunc) ephy_history_window_init
-		};
-
-		type = g_type_register_static (GTK_TYPE_WINDOW,
-					       "EphyHistoryWindow",
-					       &our_info, 0);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (EphyHistoryWindow, ephy_history_window, GTK_TYPE_WINDOW)
 
 static void
 ephy_history_window_show (GtkWidget *widget)
@@ -625,7 +597,7 @@ ephy_history_window_show (GtkWidget *widget)
 
 	gtk_widget_grab_focus (window->priv->search_entry);
 
-	GTK_WIDGET_CLASS (parent_class)->show (widget);
+	GTK_WIDGET_CLASS (ephy_history_window_parent_class)->show (widget);
 }
 
 static void
@@ -634,8 +606,6 @@ ephy_history_window_class_init (EphyHistoryWindowClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = ephy_history_window_finalize;
 
 	object_class->set_property = ephy_history_window_set_property;
@@ -674,7 +644,7 @@ ephy_history_window_finalize (GObject *object)
                          (gpointer *)window);
 	}
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (ephy_history_window_parent_class)->finalize (object);
 }
 
 static void
@@ -692,6 +662,25 @@ ephy_history_window_node_activated_cb (GtkWidget *view,
 			    EPHY_NEW_TAB_OPEN_PAGE);
 }
 
+static void
+ephy_history_window_node_middle_clicked_cb (GtkWidget *view,
+					    EphyNode *node,
+					    EphyHistoryWindow *editor)
+{
+	EphyWindow *window;
+	const char *location;
+
+	window = EPHY_WINDOW (get_target_window (editor));
+
+	location = ephy_node_get_property_string
+		(node, EPHY_NODE_PAGE_PROP_LOCATION);
+	g_return_if_fail (location != NULL);
+
+	ephy_shell_new_tab (ephy_shell, window, NULL, location,
+			    EPHY_NEW_TAB_OPEN_PAGE |
+			    EPHY_NEW_TAB_IN_EXISTING_WINDOW);
+}
+
 static void
 ephy_history_window_update_menu (EphyHistoryWindow *editor)
 {
@@ -1466,6 +1455,10 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 			  "node_activated",
 			  G_CALLBACK (ephy_history_window_node_activated_cb),
 			  editor);
+	g_signal_connect (G_OBJECT (pages_view),
+			  "node-middle-clicked",
+			  G_CALLBACK (ephy_history_window_node_middle_clicked_cb),
+			  editor);
 	g_signal_connect (G_OBJECT (pages_view),
 			  "popup_menu",
 			  G_CALLBACK (ephy_history_window_show_popup_cb),
@@ -1626,5 +1619,5 @@ ephy_history_window_dispose (GObject *object)
 		save_date_filter (editor);
 	}
 
-	G_OBJECT_CLASS (parent_class)->dispose (object);
+	G_OBJECT_CLASS (ephy_history_window_parent_class)->dispose (object);
 }
