@@ -14,9 +14,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-toolbar-editor.c 6740 2006-12-15 14:01:58Z chpe $
+ *  $Id$
  */
 
 #include "config.h"
@@ -48,7 +48,6 @@
 #include <string.h>
 
 #define DATA_KEY		"EphyToolbarEditor"
-#define CONTROL_CENTRE_DOMAIN	"control-center-2.0"
 
 #define EPHY_TOOLBAR_EDITOR_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_TOOLBAR_EDITOR, EphyToolbarEditorPrivate))
 
@@ -82,10 +81,10 @@ toolbar_styles [] =
 	   * the correct translation. You MUST OMIT it in the translated string. */
 	  N_("toolbar style|Default"), 0, FALSE },
 	{ NULL /* separator row */, 0, FALSE },
-	{ "Text below icons", EGG_TB_MODEL_BOTH, TRUE },
-	{ "Text beside icons", EGG_TB_MODEL_BOTH_HORIZ, TRUE },
-	{ "Icons only", EGG_TB_MODEL_ICONS, TRUE },
-	{ "Text only", EGG_TB_MODEL_TEXT, TRUE }
+	{ N_("Text below icons"), EGG_TB_MODEL_BOTH, TRUE },
+	{ N_("Text beside icons"), EGG_TB_MODEL_BOTH_HORIZ, TRUE },
+	{ N_("Icons only"), EGG_TB_MODEL_ICONS, TRUE },
+	{ N_("Text only"), EGG_TB_MODEL_TEXT, TRUE }
 };
 
 enum
@@ -94,7 +93,7 @@ enum
 	PROP_WINDOW
 };
 
-static GObjectClass *parent_class = NULL;
+G_DEFINE_TYPE (EphyToolbarEditor, ephy_toolbar_editor, GTK_TYPE_DIALOG)
 
 static gboolean
 row_is_separator (GtkTreeModel *model,
@@ -181,14 +180,9 @@ ephy_toolbar_editor_constructor (GType type,
 	char *pref;
 	int i;
 
-	object = parent_class->constructor (type, n_construct_properties,
-					    construct_params);
-
-#ifdef ENABLE_NLS
-	/* Initialize the control centre domain */
-	bindtextdomain (CONTROL_CENTRE_DOMAIN, GNOMELOCALEDIR);
-	bind_textdomain_codeset(CONTROL_CENTRE_DOMAIN, "UTF-8");
-#endif
+	object = G_OBJECT_CLASS (ephy_toolbar_editor_parent_class)->constructor (type,
+                                                                                 n_construct_properties,
+                                                                                 construct_params);
 
 	dialog = GTK_WIDGET (object);
 	priv = EPHY_TOOLBAR_EDITOR (object)->priv;
@@ -229,11 +223,7 @@ ephy_toolbar_editor_constructor (GType type,
 		const char *text = toolbar_styles[i].text;
 		const char *tr_text = NULL;
 
-		if (toolbar_styles[i].cc_domain)
-		{
-			tr_text = dgettext (CONTROL_CENTRE_DOMAIN, text);
-		}
-		else if (text != NULL)
+		if (text != NULL)
 		{
 			tr_text= Q_(text);
 		}
@@ -316,7 +306,7 @@ ephy_toolbar_editor_finalize (GObject *object)
 
 	g_object_set_data (G_OBJECT (priv->window), DATA_KEY, NULL);
 
-	parent_class->finalize (object);
+	G_OBJECT_CLASS (ephy_toolbar_editor_parent_class)->finalize (object);
 }
 
 static void
@@ -351,8 +341,6 @@ ephy_toolbar_editor_class_init (EphyToolbarEditorClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkDialogClass *dialog_class = GTK_DIALOG_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->constructor = ephy_toolbar_editor_constructor;
 	object_class->finalize = ephy_toolbar_editor_finalize;
 	object_class->get_property = ephy_toolbar_editor_get_property;
@@ -372,34 +360,6 @@ ephy_toolbar_editor_class_init (EphyToolbarEditorClass *klass)
 	g_type_class_add_private (object_class, sizeof (EphyToolbarEditorPrivate));
 }
 
-GType
-ephy_toolbar_editor_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (EphyToolbarEditorClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) ephy_toolbar_editor_class_init,
-			NULL,
-			NULL,
-			sizeof (EphyToolbarEditor),
-			0,
-			(GInstanceInitFunc) ephy_toolbar_editor_init
-		};
-
-		type = g_type_register_static (GTK_TYPE_DIALOG,
-					       "EphyToolbarEditor",
-					       &our_info, 0);
-	}
-
-	return type;
-}
-
 GtkWidget *
 ephy_toolbar_editor_show (EphyWindow *window)
 {
